/*global module, require, sails */

module.exports = {
    getConnections: function (params, cb) {
        'use strict';
		// first get all room names
		function getRoomNames (next) {
			sails.hooks.easyrtc.appObj.getRoomNames(function (err, roomNames) {
				if (err) {
					return next(err);
				}
				
				return next(null, roomNames);
			});
		}
		
		// second - process all rooms
		function processRooms (roomNames, next) {
			var connections = [];
			// this is helper function - will be called for each room object, returned by the async.each method
			function processRoom (roomObj, roomProcessed) {
				// get room occupants for given room object
				function getRoomOccupants (done) {
					roomObj.getField("roomOccupants", function (err, roomOccupants) {
						if (err) {
							return done(err);
						}
						
						return done(null, roomOccupants.fieldValue);
					});
				}
				
				// get room connections for given room object
				function getRoomConnections (done) {
					roomObj.getConnectionObjects(function (err, connObjects) {
						if (err) {
							return done(err);
						}
						
						return done(null, connObjects);
					});
				}
				
				// when room occupants and room connections are extracted - use them to fill connections array
				function roomDone (err, results) {
					if (err) {
						return roomProcessed(err);
					}
					
					var roomOccupants = results[0],
						roomConnectionObjects = results[1];
					
					for (var i = 0; i < roomConnectionObjects.length; i++) {
						if ((roomOccupants.users.indexOf(roomConnectionObjects[i].getEasyrtcid()) !== -1 
							|| roomOccupants.guests.indexOf(roomConnectionObjects[i].getEasyrtcid()) !== -1)
							&& roomConnectionObjects[i].getUsername().toLowerCase().indexOf(params.search.toLowerCase()) !== -1) {
							// ok, connection is either in users or guests array - so push it into connections array
							connections.push({easyrtcId: roomConnectionObjects[i].getEasyrtcid(), name: roomConnectionObjects[i].getUsername()});
						}
					}
					
					return roomProcessed(null);
				}
				
				// get occupants and connections in parallel, just to save time
				async.parallel([getRoomOccupants, getRoomConnections], roomDone);
			}
			
			// this is called by async.each - will extract room object for given roomName
			function getRoomObj (roomName, done) {
				sails.hooks.easyrtc.appObj.room(roomName, function (err, roomObj) {
					if (err) {
						return done(err);
					}
					
					processRoom (roomObj, done);
				});
			}

			// all roomNames array elements are processed - return connections to the upper level
			function allRoomsDone (err) {
				// remove duplicated values, compare by easyrtcId field
				connections = _.unique(connections, 'easyrtcId');
				return next(err, connections);
			}
			
			// iterate over roomNames array
			async.each(roomNames, getRoomObj, allRoomsDone);
		}
		
		// third - when all is done, return connections to the client
		function allDone (err, connections) {
			if (err) {
				return cb(err);
			}
			
			return cb(null, connections);
		}
		
		// fire up the process
		async.waterfall([getRoomNames, processRooms], allDone);
    },
    notifyOnRoomLeave: function (params, cb) {
		function getConnections (next) {
			sails.hooks.easyrtc.appObj.getConnectionEasyrtcids(function (err, connections) {
				if (err) {
					return next(err);
				}
				
				return next(null, connections);
			});
		}
		
		function processConnections (connections, next) {
			function getConnectionObj (easyrtcId, done) {
				sails.hooks.easyrtc.appObj.connection(easyrtcId, function (err, connectionObj) {
					if (err) {
						// if connection is not valid, just pass it, without throwing any error
						return done(null);
					}
					// build return message
					var returnMsg = {
						"msgType": "notifyOnRoomLeave",
						"msgData": {
							"leavingConnection": params.leavingConnection
						}
					};
					sails.hooks.easyrtc.appObj.events.emit("emitEasyrtcMsg", connectionObj, "notifyOnRoomLeave", returnMsg, null, sails.hooks.easyrtc.appObj.util.nextToNowhere);
					done(null);
				});
			}
			
			function allConnectionsDone (err) {
				return next(err);
			}
			
			// iterate over connections array
			async.each(connections, getConnectionObj, allConnectionsDone);
		}
		
		function allDone (err) {
			if (err) {
				return cb(err);
			}

			return cb(null);
		}
		
		async.waterfall([getConnections, processConnections], allDone);
	}
};